<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Boat;
use App\Models\BoatCategory;
use App\Models\Image;
use App\Models\Location;
use App\Rules\FileTypeValidate;
use Illuminate\Http\Request;


class BoatController extends Controller
{
    public function index()
    {
        $pageTitle                  = 'Boat Information';
        $boats                      = Boat::with('category', 'location')->orderBy('id', 'desc')->paginate(getPaginate(10));
        return view('admin.boat.index', compact('pageTitle', 'boats'));
    }
    public function create()
    {    
        $pageTitle                  = 'Boat Create';
        $locations                  = Location::where('status', 1)->get();
        $categories                 = BoatCategory::where('status', 1)->get();
        return view('admin.boat.create', compact('pageTitle', 'locations', 'categories'));
    }

    public function store(Request $request)
    {
        $request->validate([
            'name'                  => 'required|string|max:255',
            'mileage'               => 'required|numeric|gt:0',
            'category'              => 'required|exists:boat_categories,id',
            'location'              => 'required|exists:locations,id',
            'price'                 => 'required|numeric|gt:0',
            'passengers'            => 'required|numeric|gt:0',
            'fuel'                  => 'required',
            'class'                 => 'required',
            'description'           => 'required',
            'additonal.*'           => 'required',
            'size'                  => 'required|numeric|gt:0',
            'images.*'              => ['required', 'image', new FileTypeValidate(['jpg', 'jpeg', 'png'])],
        ]);


        $purifier                   = new \HTMLPurifier();
        $boat                       = new Boat();
        $boat->name                 = $request->name;
        $boat->category_id          = $request->category;
        $boat->location_id          = $request->location;
        $boat->price                = $request->price;
        $boat->passengers           = $request->passengers;
        $boat->class                = $request->class;
        $boat->fuel                 = $request->fuel;
        $boat->mileage              = $request->mileage;
        $boat->description          = $purifier->purify($request->description);
        $boat->cabins               = $request->cabin;
        $boat->berths               = $request->berths;
        $boat->bathrooms            = $request->bathrooms;
        $boat->size                 = $request->size;
        $boat->additional           = json_encode($request->additonal);

        if($request->captain)
        {
            $boat->is_captain       = $request->captain;
        }
        else{
            $boat->is_captain       = 0;
        }
        $boat->save();

        if($request->images){
            foreach($request->images as $image)
            {
                $newImage           = new Image();   
                $newImage->boat_id  = $boat->id;
                try {
                    $newImage->image = fileUploader($image, getFilePath('boatImage'),getFileSize('boatImage'));
                } catch (\Exception $exp) {
                    $notify[]       = ['error', 'Couldn\'t upload your image'];
                    return back()->withNotify($notify);
                }
                $newImage->save();
            }
        }

        $notify[]                   = ['success', 'Location Created successfully.'];
        return to_route('admin.boat.list')->withNotify($notify);
    }
    public function edit($id)
    {    
        $pageTitle              = 'Boat Edit';
        $boat                   = Boat::findOrFail($id);
        $locations              = Location::where('status', 1)->get();
        $categories             = BoatCategory::where('status', 1)->get();
        $images                 = Image::where('boat_id', $boat->id)->get();
        return view('admin.boat.edit', compact('pageTitle', 'boat', 'locations', 'categories', 'images'));
    }
    public function update($id, Request $request)
    {

        $request->validate([
            'name'              => 'required|string|max:255',
            'mileage'           => 'required|numeric|gt:0',
            'category'          => 'required|exists:boat_categories,id',
            'location'          => 'required|exists:locations,id',
            'price'             => 'required|numeric|gt:0',
            'passengers'        => 'required|numeric|gt:0',
            'fuel'              => 'required',
            'class'             => 'required',
            'additonal.*'       => 'required',
            'size'              => 'required|numeric|gt:0',
            'description'       => 'string', 
            'images.*'          => ['nullable', 'image', new FileTypeValidate(['jpg', 'jpeg', 'png'])],
        ]);

        $boat                       = Boat::findOrFail($id);
        $purifier                   = new \HTMLPurifier();
        $boat->name                 = $request->name;
        $boat->category_id   = $request->category;
        $boat->location_id        = $request->location;
        $boat->price                = $request->price;
        $boat->passengers           = $request->passengers;
        $boat->class                = $request->class;
        $boat->fuel                 = $request->fuel;
        $boat->mileage              = $request->mileage;
        $boat->description          = $purifier->purify($request->description);
        $boat->cabins               = $request->cabin;
        $boat->berths               = $request->berths;
        $boat->size                 = $request->size;
        $boat->bathrooms            = $request->bathrooms;

        $boat->additional = json_encode($request->additonal);
        if($request->captain)
        {
            $boat->is_captain   = $request->captain;
        }
        else{
            $boat->is_captain   = 0;
        }
        $boat->save();    

        if($request->images){
            foreach($request->images as $image)
            {
                $newImage       = new Image();   
                $newImage->boat_id = $boat->id;
                try {
                    $newImage->image = fileUploader($image, getFilePath('boatImage'),getFileSize('boatImage'));
                } catch (\Exception $exp) {
                    $notify[]   = ['error', 'Couldn\'t upload your image'];
                    return back()->withNotify($notify);
                }
                $newImage->save();
            }
        }

        $notify[]               = ['success', 'Boat updated successfully.'];
        return to_route('admin.boat.list')->withNotify($notify);
    }



    public function status($id)
    {
        $location               = Boat::findOrFail($id);
        if($location->status == 1)
        {
            $location->status   = 0;
            $location->save();
            
            $notify[]           = ['error', 'Boat status successfully changed.'];
            return to_route('admin.boat.list')->withNotify($notify);
        }else{
            $location->status   = 1;
            $location->save();

            $notify[]           = ['success', 'Boat status successfully changed.'];
            return to_route('admin.boat.list')->withNotify($notify);
        }

    }


    public function imageDelete(Request $request)
    {
        $id                     = $request->id;
        $image                  = Image::findOrFail($id);
        $path                   = getFilePath('boatImage');
        fileManager()->removeFile($path.'/'.$image->image);
        $image->delete();

        $notify[]               = ['success', 'Boat image successfully deleted.'];
        return redirect()->back()->withNotify($notify);

    }
}
