<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller;
use App\Lib\FormProcessor;
use App\Lib\GoogleAuthenticator;
use App\Models\Booking;
use App\Models\Form;
use App\Models\Transaction;
use App\Rules\FileTypeValidate;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;

class UserController extends Controller
{
    public function home()
    {
        $pageTitle                          = 'Dashboard';
        $userBookingReport                  = Booking::where('user_id', auth()->user()->id)->selectRaw("SUM(price) as price, DATE(created_at) as date_name")->orderBy('created_at', 'desc')
                ->groupByRaw("DATE(created_at)")
                ->pluck('price', 'date_name');

        $userBookingReport['labels']        = $userBookingReport->keys();
        $userBookingReport['values']        = $userBookingReport->values();

        $now = date('Y-m-d');

        $userCurrentJourney                 = Booking::where('user_id', auth()->user()->id)->whereDate('from_date','<=', $now)->whereDate('to_date', '>=', $now)->get()->count();
        $userUpcommingJourney               = Booking::where('user_id', auth()->user()->id)->whereDate('from_date', '>', $now)->get()->count();
        $userPreviousJourney                = Booking::where('user_id', auth()->user()->id)->whereDate('to_date', '<', $now)->get()->count();

        return view($this->activeTemplate . 'user.dashboard', compact('pageTitle', 'userBookingReport', 'userCurrentJourney', 'userUpcommingJourney', 'userPreviousJourney'));
    }

    public function imageUpdate(Request $request)
    {
        $this->validate($request, [
            'image'                         => ['nullable','image',new FileTypeValidate(['jpg','jpeg','png'])]
        ]);
        $user = auth()->user();

        if ($request->hasFile('image')) {
            try {
                $old                        = $user->image;
                $user->image                = fileUploader($request->image, getFilePath('userProfile'), getFileSize('userProfile'), $old);
            } catch (\Exception $exp) {
                $notify[]                   = ['error', 'Couldn\'t upload your image'];
                return back()->withNotify($notify);
            }
        }
        $user->save();

        $notify[]                           = ['success', 'Profile image has been updated successfully'];
        return to_route('user.home')->withNotify($notify);
    }

    public function depositHistory(Request $request)
    {
        $pageTitle                          = 'Deposit History';
        $deposits                           = auth()->user()->deposits();
        $deposits                           = $deposits->with(['gateway'])->orderBy('id','desc')->paginate(getPaginate());
   
        return view($this->activeTemplate.'user.deposit_history', compact('pageTitle', 'deposits'));
    }

    public function show2faForm()
    {
        $general                            = gs();
        $ga                                 = new GoogleAuthenticator();
        $user                               = auth()->user();
        $secret                             = $ga->createSecret();
        $qrCodeUrl                          = $ga->getQRCodeGoogleUrl($user->username . '@' . $general->site_name, $secret);
        $pageTitle                          = '2FA Setting';

        return view($this->activeTemplate.'user.twofactor', compact('pageTitle', 'secret', 'qrCodeUrl'));
    }

    public function create2fa(Request $request)
    {
        $user                               = auth()->user();
        $this->validate($request, [
            'key'                           => 'required',
            'code'                          => 'required',
        ]);
        $response = verifyG2fa($user,$request->code,$request->key);
        if ($response) {
            $user->tsc                      = $request->key;
            $user->ts                       = 1;
            $user->save();

            $notify[]                       = ['success', 'Google authenticator activated successfully'];
            return back()->withNotify($notify);
        } else {
            $notify[]                       = ['error', 'Wrong verification code'];
            return back()->withNotify($notify);
        }
    }

    public function disable2fa(Request $request)
    {
        $this->validate($request, [
            'code'                          => 'required',
        ]);

        $user = auth()->user();
        $response = verifyG2fa($user,$request->code);
        if ($response) {
            $user->tsc                      = null;
            $user->ts                       = 0;
            $user->save();
            $notify[]                       = ['success', 'Two factor authenticator deactivated successfully'];
        } else {
            $notify[]                       = ['error', 'Wrong verification code'];
        }
        return back()->withNotify($notify);
    }

    public function bookingHistory()
    {
        $pageTitle                          = 'Booking History';
        $bookings                           = Booking::with('getLocation')->where('user_id', auth()->user()->id)->orderBy('id','desc')->paginate(getPaginate(10));
        return view($this->activeTemplate.'user.bookings', compact('pageTitle','bookings'));
    }

    public function transactions(Request $request)
    {
        $pageTitle                          = 'Transactions';
        // $remarks                         = Transaction::distinct('remark')->orderBy('remark')->get('remark');
        $transactions                       = Transaction::where('user_id',auth()->id());

        if ($request->search) {
            $transactions = $transactions->where('trx',$request->search);
        }

        // if ($request->type) {
        //     $transactions = $transactions->where('trx_type',$request->type);
        // }

        // if ($request->remark) {
        //     $transactions = $transactions->where('remark',$request->remark);
        // }

        $transactions                       = $transactions->orderBy('id','desc')->paginate(getPaginate(10));
        return view($this->activeTemplate.'user.transactions', compact('pageTitle','transactions'));
    }

    public function userData()
    {
        $user                               = auth()->user();
        if ($user->reg_step == 1) {
            return to_route('user.home');
        }
        $pageTitle                          = 'User Data';
        return view($this->activeTemplate.'user.user_data', compact('pageTitle','user'));
    }

    public function userDataSubmit(Request $request)
    {
        $user                               = auth()->user();
        if ($user->reg_step == 1) {
            return to_route('user.home');
        }
        $request->validate([
            'firstname'                     => 'required',
            'lastname'                      => 'required',
        ]);
        $user->firstname                    = $request->firstname;
        $user->lastname                     = $request->lastname;
        $user->address = [
            'country'                       => @$user->address->country,
            'address'                       => $request->address,
            'state'                         => $request->state,
            'zip'                           => $request->zip,
            'city'                          => $request->city,
        ];
        $user->reg_step                     = 1;
        $user->save();

        $notify[]                           = ['success','Registration process completed successfully'];
        return to_route('user.home')->withNotify($notify);

    }

}
